class AuthenticationError(Exception):
    """Raised when login credentials are missing"""

    message = """
    Autenthication failed.
    The required configuration setting {} was not found in your environment.
    To authenticate follow the instructions explained in the README:
    https://gitlab.com/coopdevs/pymasmovil#login
    """

    def __init__(self, missing_credential):
        self.message = self.message.format(missing_credential)
        super(AuthenticationError, self).__init__(self.message)



from pymasmovil.client import Client
from pymasmovil.errors.account_required_params_error import AccountRequiredParamsError


class Account():
    _route = '/v0/accounts'

    town = ""
    surname = ""
    stair = ""
    roadType = ""
    roadNumber = ""
    roadName = ""
    region = ""
    province = ""
    postalCode = ""
    phone = ""
    name = ""
    id = ""
    flat = ""
    email = ""
    door = ""
    documentType = ""
    documentNumber = ""
    corporateName = ""
    buildingPortal = ""

    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            if hasattr(Account, key):
                setattr(self, key, value)

    @classmethod
    def get(cls, session, account_id):
        """
        Returns a account instance obtained by id.

        :param id:
        :return: Account:
        """

        account_response = Client(session).get(
            route='{}/{}'.format(cls._route, account_id))

        return Account(**account_response)

    @classmethod
    def create(cls, session, **new_account):
        """
            Creates an account instance.

            :param **new_account:
            :return:
        """

        required_paramether_list = ['documentNumber', 'documentType', 'email',
                                    'phone', 'postalCode', 'province', 'region', 'roadName',
                                    'roadNumber', 'roadType', 'town']

        if 'documentType' not in new_account:
            raise AccountRequiredParamsError('documentType')

        if new_account['documentType'] == "CIF":
            required_paramether_list.append("corporateName")
        else:
            required_paramether_list.extend(["name", "surname", "nationality"])

        for required_paramether in required_paramether_list:
            if required_paramether not in new_account:
                raise AccountRequiredParamsError(required_paramether)

        new_account_id = Client(session).post(cls._route, (), new_account)

        return Account(id=new_account_id, **new_account)



from pymasmovil.client import Client
from pymasmovil.requests.new_line_request import NewLineRequest


class OrderItem():
    _route = '/v0'

    id = ''
    account_id = ''
    name = ''
    surname = ''
    productName = ''
    phone = ''
    orderType = ''
    status = ''
    createdDate = ''
    lastModifiedDate = ''
    attributes = {
        'Apellidos': '',
        'Fecha_Portabilidad_Saliente': '',
        'ICCID_Donante': '',
        'Nombre': '',
        'Numero_de_Documento': '',
        'Operador_Donante_Movil': '',
        'Operador_Receptor_Movil': '',
        'Tipo_de_Documento': '',
        'Tipo_de_Linea': '',
        'Fecha_de_solicitud_del_abonado': '',
        'Porcentaje_Consumo_Bono': ''
    }
    simAttributes = {
        'ICCID': '',
        'IMSI': '',
        'PIN': '',
        'PIN2': '',
        'PUK': '',
        'PUK2': ''
    }

    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            if key == 'attributes':
                for inner_key, inner_value in value.items():
                    if inner_key in self.attributes:
                        self.attributes[inner_key] = inner_value
            if key == 'simAttributes':
                for inner_key, inner_value in value.items():
                    if inner_key in self.simAttributes:
                        self.simAttributes[inner_key] = inner_value
            else:
                if hasattr(OrderItem, key):
                    setattr(self, key, value)

    @classmethod
    def get(cls, session, order_item_id):
        """
            Returns an OrderItem instance obtained by id.
            :param order_item_id:

            :return: OrderItem:
        """

        order_item = Client(session).get(
            route='{}/{}'.format(cls._route, order_item_id))

        return OrderItem(**order_item)

    @classmethod
    def create(cls, session, account, **new_order_item):
        """
            Creates an order-item and posts it to the given account

            :param account: account where we want to add the order-item
            :param **new_order_item:
            :return: order-item instance
        """

        post_route = '{}/accounts/{}/order-items'.format(cls._route, account.id)

        Client(session).post(post_route, (), new_order_item)

        new_line_request = NewLineRequest(new_order_item)

        return OrderItem(**new_line_request.to_order_item())



class AccountRequiredParamsError(Exception):
    """Raised when trying to create an account without some required paramether"""

    message = """
    .
    Missing required paramether "{}" to create an account.
    Please make sure all required parameters are provided:
    [corporateName*, documentNumber, documentType, nationality*, email,
    name*, surname*, phone, postalCode, province, region, roadName,
    roadNumber, roadType, town]
    * if appropriate, according to documentType
    """

    def __init__(self, missing_paramether):
        self.message = self.message.format(missing_paramether)
        super().__init__(self.message)
